﻿/*======================================================================*//**
 * PaperEffectCastSprite
 * Papervision3Dを利用した画面遷移を設定したCastSpriteです。
 * 動作には　Papervision3D 2.0 beta　のライブラリが必要になります。
 * またムービークリップには graphics_mc と命名したインスタンスが必要になります。
 */
package jp.clockmaker.casts {
	import jp.progression.casts.*;
	import jp.progression.commands.*;
	import jp.progression.events.*;
	import jp.progression.loader.*;
	import jp.progression.*;
	import jp.progression.scenes.*;
	import flash.display.*;
	import flash.geom.*;
	import flash.events.*;
	import org.papervision3d.cameras.*;
	import org.papervision3d.events.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.materials.utils.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.objects.primitives.*;
	import org.papervision3d.render.*;
	import org.papervision3d.scenes.*;
	import org.papervision3d.view.*;
	
	/*======================================================================*//**
	 * PaperEffectCastSprite クラス
	 */
	public class PaperEffectCastSprite extends CastSprite {
		
		// タイルの分割数
		static private const MAX_CELL_W:uint = 8;
		static private const MAX_CELL_H:uint = 5;

		private var scene    :Scene3D;
		private var viewport :Viewport3D;
		private var camera   :Camera3D;
		private var renderer :BasicRenderEngine;
		
		// 細切れになった写真の3Dパーツを保存する配列です
		private var planeArr:Array  = [];
		
		/*======================================================================*//**
		 * コンストラクタ
		 */
		public function PaperEffectCastSprite( initObject:Object = null ) {
			super( initObject );
			
			//　3Dの初期設定を行います
			scene     = new Scene3D();
			viewport  = new Viewport3D(graphics_mc.width, graphics_mc.height, false, false);
			camera    = new Camera3D(60, 10, 500);
			renderer  = new BasicRenderEngine();
			
			viewport.x = graphics_mc.x;
			viewport.y = graphics_mc.y;
			addChild(viewport);
			
			camera.focus = 100;
			camera.zoom  = 2;
			
			// 画像を分割キャプチャします
			var captureBmpData:BitmapData =  new BitmapData(graphics_mc.width, graphics_mc.height, true, 0x00000000);
			captureBmpData.draw(graphics_mc);
			var bitmap:Bitmap = new Bitmap(captureBmpData);
			
			var bmpDataArr:Array  = [];
			
			// 二重ループを開始します
			var i:int = 0;
			var j:int = 0;
			
			for (i = 0; i < MAX_CELL_W; i++ ){
				for (j = 0; j < MAX_CELL_H; j++ ){
					var bmpData:BitmapData = new BitmapData(bitmap.width / MAX_CELL_W, bitmap.height/ MAX_CELL_H, true, 0x00000000);
					
					// 行列を用いてキャプチャする画像の原点をずらします
					var matrix:Matrix = new Matrix();
					matrix.translate( - bitmap.width / MAX_CELL_W * i, - bitmap.height / MAX_CELL_H * j);
			
					// 　行列変換を用いてキャプチャします
					bmpData.draw(bitmap, matrix);
					bmpDataArr.push(bmpData);
				}
			}
			
			// 分割平面を作成します
			var wrap:DisplayObject3D = scene.addChild(new DisplayObject3D());
			wrap.x = - bitmap.width / 2;
			wrap.y = - bitmap.height / 2;
			
			for (i = 0; i < MAX_CELL_W; i++ ){
				planeArr[i] = [];
				for (j = MAX_CELL_H; j > 0 ; j-- ){
					var material:BitmapMaterial = new BitmapMaterial(BitmapData(bmpDataArr.shift()));
					
					var plane:Plane = new Plane(material, bitmap.width / MAX_CELL_W, bitmap.height / MAX_CELL_H, 1, 1);
					plane.x = bitmap.width / MAX_CELL_W * i + bitmap.width / MAX_CELL_W / 2;
					plane.y = bitmap.height / MAX_CELL_H * j - bitmap.height / MAX_CELL_H / 2;
					
					planeArr[i][j] = wrap.addChild(plane);
				}
			}
		}
		
		/*======================================================================*//**
		 * ICastObject オブジェクトが AddChild コマンド、または AddChildAt コマンド経由でディスプレイリストに追加された場合に送出されます。
		 * このイベント処理の実行中には、addCommand() メソッド、及び insertCommand() メソッドによるコマンドの同期処理が行えます。
		 */
		protected override function _onCastAdded():void {
			// 実行したいコマンドを登録します。
			addCommand(
				// 3Dレイヤーを表示します
				new Prop(viewport, {visible:true}),
				// 遷移先画面を非表示にします
				new Prop(graphics_mc, {visible:false}),
				// レンダリングを開始します
				function():void{
					addEventListener(Event.ENTER_FRAME, doRender);
				},
				// 演出のコマンドを作成します
				function():void{
					// 演出は同時に行って欲しいので、並列コマンドを作成します
					var showCommand:ParallelList = new ParallelList();
					
					// 二重ループを開始します
					var i:int = 0;
					var j:int = 0;
					
					// 分割平面のモーションを設定します
					for (i = 0; i < planeArr.length; i++ ){
						for (j = 1; j < planeArr[i].length ; j++ ){
							var plane:Plane = Plane(planeArr[i][j]);
							var distance:Number = Math.sqrt(i * i + j * j);
							
							showCommand.addCommand(
								new Prop(plane, {
									z          : -500,
									rotationY  : 360,
									scale      : 1
								}),
								new DoTweener(plane, {
									z          : 0,
									time       : 1.2,
									transition : "easeOutExpo",
									delay      : distance * .1
								}),
								new DoTweener(plane, {
									rotationY  : 0,
									time       : .7,
									transition : "easeOutExpo",
									delay      : distance * .08  + .8
								})
							);
						}
					}
					
					// カメラのモーションを設定します
					showCommand.addCommand(
						new Prop(camera,{
							x:0, y:0, z:-500
						}),
						new DoTweener(camera,{
							x:0, y:0, z:-200,
							time       : 1,
							transition : "easeInCubic",
							delay      : 1.1
						})
					)
					
					// 演出コマンドを追加します
					this.parent.insertCommand(showCommand);
				},
				// この時点で演出が終了しているのでレンダリングを停止します
				function():void {
					removeEventListener(Event.ENTER_FRAME, doRender);
				},
				// 演出が終了しているので画面を表示します
				new Prop(graphics_mc, {visible:true}),
				// 演出が終了しているので演出用の3Dレイヤーを非表示にします
				new Prop(viewport, {visible:false})
			);
		}
		
		/*======================================================================*//**
		 * ICastObject オブジェクトが RemoveChild コマンド、または RemoveAllChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		 * このイベント処理の実行中には、addCommand() メソッド、及び insertCommand() メソッドによるコマンドの同期処理が行えます。
		 */
		protected override function _onCastRemoved():void {
			// 実行したいコマンドを登録します。
			addCommand(
				new Prop(viewport, {visible:true}),
				new Prop(graphics_mc, {visible:false}),
				function():void {
					addEventListener(Event.ENTER_FRAME, doRender);
				},
				function():void {
					var hideCommand:ParallelList = new ParallelList();
					
					// 二重ループを開始します
					var i:int = 0;
					var j:int = 0;
					
					// 分割平面のモーションを設定します
					for (i = 0; i < planeArr.length; i++ ){
						for (j = 1; j < planeArr[i].length ; j++ ){
							var plane:Plane = Plane(planeArr[i][j]);
							
							var distance:Number = Math.sqrt(i * i + j * j);
							
							hideCommand.addCommand(
								new DoTweener(plane, {
									z          : 500,
									time       : .9,
									scale      : 0,
									transition : "easeInQuad",
									delay      : distance * .08 + .2
								}),
								new DoTweener(plane, {
									rotationY  : 360,
									time       : .8,
									transition : "easeInOutSine",
									delay      : distance * .08
								})
							);
						}
					}
					
					// カメラのモーションを設定します
					hideCommand.addCommand(
						new Prop(camera, {
							x:0, y:0, z:-200
						})
					)
					
					this.parent.insertCommand(hideCommand);
				},
				function():void {
					removeEventListener(Event.ENTER_FRAME, doRender);
				},
				new Prop(viewport, {visible:false})
			);
		}
		
		/*======================================================================*//**
		* レンダリング処理のメソッドです
		*/
		private function doRender(e:Event):void {
			renderer.renderScene(scene, camera, viewport);
		}
	}
}









